@@ -10,13 +10,17 @@
 #' @param gp_model select between "simple", "none" to apply a model for whether a player played in a given game, defaults to "simple"
 #' @param base_seasons a numeric vector that selects seasons as base data, earliest available is 2012
 #' @param actual_schedule a logical: use actual ff_schedule? default is FALSE
+#' @param replacement_level a logical: use best available on waiver as  replacement level? defaults to TRUE
 #' @param pos_filter a character vector of positions to filter/run, default is c("QB","RB","WR","TE","K")
 #' @param verbose a logical: print status messages? default is TRUE, configure with options(ffsimulator.verbose)
+#' @param return one of c("default", "all") - what objects to return in the output list
 #'
 #' @examples
 #' \donttest{
+#' try({ # try block to prevent CRAN-related issues
 #' conn <- mfl_connect(2021, 22627)
 #' ff_simulate(conn, n_seasons = 25)
+#' })
 #' }
 #'
 #' @return an `ff_simulation` object which can be passed to `plot()` and contains the output data from the simulation.
@@ -33,8 +37,29 @@ ff_simulate <- function(conn,
                         gp_model = c("simple", "none"),
                         base_seasons = 2012:2020,
                         actual_schedule = FALSE,
+                        replacement_level = TRUE,
                         pos_filter = c("QB","RB","WR","TE","K"),
-                        verbose = getOption("ffsimulator.verbose", default = TRUE)) {
+                        verbose = NULL,
+                        return = c("default", "all")
+) {
+
+  #### TEST ####
+
+  # conn <- mfl_connect(2021,54040)
+  # conn <- sleeper_connect(2021,"734442977157603328")
+  # verbose <- NULL
+  # base_seasons = 2012:2020
+  # gp_model = "simple"
+  # pos_filter = c("QB","RB","WR","TE","K")
+  # n_seasons = 100
+  # n_weeks = 14
+  # best_ball = FALSE
+  # seed = NULL
+  # base_seasons = 2012:2020
+  # actual_schedule = TRUE
+  # pos_filter = c("QB","RB","WR","TE","K")
+  # verbose = TRUE
+  # return = "all"
 
   #### Assertions ####
 
@@ -44,7 +69,8 @@ ff_simulate <- function(conn,
     )
   }
 
-  gp_model <- rlang::arg_match0(gp_model,c("simple","none"))
+  gp_model <- rlang::arg_match0(gp_model, c("simple","none"))
+  return <- rlang::arg_match0(return, c("default","all"))
   checkmate::assert_subset(pos_filter, c("QB","RB","WR","TE","K"))
   checkmate::assert_numeric(base_seasons, lower = 2012, upper = 2020)
   checkmate::assert_int(n_seasons, lower = 1)
@@ -52,12 +78,9 @@ ff_simulate <- function(conn,
   checkmate::assert_int(seed, null.ok = TRUE)
   if (!is.null(seed)) set.seed(seed)
   checkmate::assert_flag(best_ball)
-  checkmate::assert_flag(verbose)
+  if(!is.null(verbose)) set_verbose(verbose)
   checkmate::assert_flag(actual_schedule)
-
-  vcli_start <- verbose_cli(verbose, "start")
-  vcli_end <- verbose_cli(verbose, "end")
-  vcli_rule <- verbose_cli(verbose,"rule")
+  checkmate::assert_flag(replacement_level)
 
   #### Import Data ####
 
@@ -115,6 +138,16 @@ ff_simulate <- function(conn,
 
   vcli_start(msg = "Generating Projections")
 
+  if(!replacement_level) rosters_rl <- rosters
+
+  if(replacement_level){
+    rosters_rl <- ffs_add_replacement_level(rosters = rosters,
+                                            latest_rankings = latest_rankings,
+                                            franchises = franchises,
+                                            lineup_constraints = lineup_constraints,
+                                            pos_filter = pos_filter)
+  }
+
   adp_outcomes <- ffs_adp_outcomes(
     scoring_history = scoring_history,
     gp_model = gp_model,
@@ -126,20 +159,16 @@ ff_simulate <- function(conn,
     latest_rankings = latest_rankings,
     n_seasons = n_seasons,
     weeks = weeks,
-    rosters = rosters
+    rosters = rosters_rl
   )
 
-  # filter out played game weeks
-
-
   vcli_end(msg_done = "Generating Projections...done! {Sys.time()}")
 
-  #### Calculate Roster Scores ####
   vcli_start(msg = "Calculating Roster Scores")
 
   roster_scores <- ffs_score_rosters(
     projected_scores = projected_scores,
-    rosters = rosters
+    rosters = rosters_rl
   )
 
   vcli_end(msg_done = "Calculating Roster Scores...done! {Sys.time()}")
@@ -155,8 +184,6 @@ ff_simulate <- function(conn,
 
   vcli_end(msg = "Optimizing Lineups...done! {Sys.time()}")
 
-  #### Generate Schedules ####
-
   vcli_start(msg = "Building Schedules")
 
   if(actual_schedule) {
@@ -174,7 +201,6 @@ ff_simulate <- function(conn,
 
   vcli_end(msg_done = "Building Schedules...done! {Sys.time()}")
 
-  #### Summarise Season ####
   vcli_start(msg = "Summarising Simulation Data")
 
   summary_week <- ffs_summarise_week(optimal_scores, schedules)
@@ -183,30 +209,64 @@ ff_simulate <- function(conn,
 
   vcli_end(msg_done = "Summarising Simulation Data...done! {Sys.time()}")
 
-  #### Build and Return ####
+  if(return == "default"){
 
-  out <- structure(
-    list(
-      summary_simulation = summary_simulation,
-      summary_season = summary_season,
-      summary_week = summary_week,
-      roster_scores = roster_scores,
-      projected_scores = projected_scores,
-      league_info = league_info,
-      simulation_params = list(
-        n_seasons = n_seasons,
-        n_weeks = n_weeks,
-        scrape_date = latest_rankings$scrape_date[[1]],
-        best_ball = best_ball,
-        seed = seed,
-        gp_model = gp_model,
-        actual_schedule = actual_schedule,
-        base_seasons = list(base_seasons),
-        pos_filter = list(pos_filter)
-      )
-    ),
-    class = "ff_simulation"
-  )
+    out <- structure(
+      list(
+        summary_simulation = summary_simulation,
+        summary_season = summary_season,
+        summary_week = summary_week,
+        roster_scores = roster_scores,
+        projected_scores = projected_scores,
+        league_info = league_info,
+        simulation_params = list(
+          n_seasons = n_seasons,
+          n_weeks = n_weeks,
+          scrape_date = latest_rankings$scrape_date[[1]],
+          best_ball = best_ball,
+          seed = seed,
+          gp_model = gp_model,
+          actual_schedule = actual_schedule,
+          base_seasons = list(base_seasons),
+          pos_filter = list(pos_filter)
+        )
+      ),
+      class = "ff_simulation"
+    )
+  }
+
+  if(return == "all"){
+
+    out <- structure(
+      list(
+        summary_simulation = summary_simulation,
+        summary_season = summary_season,
+        summary_week = summary_week,
+        schedules = schedules,
+        optimal_scores = optimal_scores,
+        roster_scores = roster_scores,
+        projected_scores = projected_scores,
+        scoring_history = scoring_history,
+        franchises = franchises,
+        rosters = rosters,
+        lineup_constraints = lineup_constraints,
+        latest_rankings = latest_rankings,
+        league_info = league_info,
+        simulation_params = list(
+          n_seasons = n_seasons,
+          n_weeks = n_weeks,
+          scrape_date = latest_rankings$scrape_date[[1]],
+          best_ball = best_ball,
+          seed = seed,
+          gp_model = gp_model,
+          actual_schedule = actual_schedule,
+          base_seasons = list(base_seasons),
+          pos_filter = list(pos_filter)
+        )
+      ),
+      class = "ff_simulation"
+    )
+  }
 
   vcli_rule("Simulation complete! {Sys.time()}")
 
@@ -226,17 +286,3 @@ print.ff_simulation <- function(x, ...) {
   str(x, max.level = 1, give.attr = FALSE)
   invisible(x)
 }
-
-dump_function <- function(...) NULL
-
-verbose_cli <- function(verbose, type) {
-  if (!verbose) return(dump_function)
-
-  if (type == "start") return(cli::cli_process_start)
-
-  if (type == "end") return(cli::cli_process_done)
-
-  if (type == "rule") return(cli::cli_rule)
-
-  return(NULL)
-}
